/* This file was generated by upb_generator from the input file:
 *
 *     opencensus/proto/trace/v1/trace_config.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "opencensus/proto/trace/v1/trace_config.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

static const upb_MiniTableSub opencensus_proto_trace_v1_TraceConfig_submsgs[3] = {
  {.UPB_PRIVATE(submsg) = &opencensus__proto__trace__v1__ProbabilitySampler_msg_init},
  {.UPB_PRIVATE(submsg) = &opencensus__proto__trace__v1__ConstantSampler_msg_init},
  {.UPB_PRIVATE(submsg) = &opencensus__proto__trace__v1__RateLimitingSampler_msg_init},
};

static const upb_MiniTableField opencensus_proto_trace_v1_TraceConfig__fields[7] = {
  {1, UPB_SIZE(12, 48), -9, 0, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 48), -9, 1, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 48), -9, 2, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, 16, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {5, 24, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {6, 32, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {7, 40, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable opencensus__proto__trace__v1__TraceConfig_msg_init = {
  &opencensus_proto_trace_v1_TraceConfig_submsgs[0],
  &opencensus_proto_trace_v1_TraceConfig__fields[0],
  UPB_SIZE(48, 56), 7, kUpb_ExtMode_NonExtendable, 7, UPB_FASTTABLE_MASK(56), 0,
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x003000080100000a, &upb_pom_1bt_max64b},
    {0x0030000802010012, &upb_pom_1bt_max64b},
    {0x003000080302001a, &upb_pom_1bt_max64b},
    {0x001000003f000020, &upb_psv8_1bt},
    {0x001800003f000028, &upb_psv8_1bt},
    {0x002000003f000030, &upb_psv8_1bt},
    {0x002800003f000038, &upb_psv8_1bt},
  })
};

static const upb_MiniTableField opencensus_proto_trace_v1_ProbabilitySampler__fields[1] = {
  {1, 8, 0, kUpb_NoSub, 1, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable opencensus__proto__trace__v1__ProbabilitySampler_msg_init = {
  NULL,
  &opencensus_proto_trace_v1_ProbabilitySampler__fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000009, &upb_psf8_1bt},
  })
};

static const upb_MiniTableField opencensus_proto_trace_v1_ConstantSampler__fields[1] = {
  {1, 8, 0, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable opencensus__proto__trace__v1__ConstantSampler_msg_init = {
  NULL,
  &opencensus_proto_trace_v1_ConstantSampler__fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv4_1bt},
  })
};

static const upb_MiniTableField opencensus_proto_trace_v1_RateLimitingSampler__fields[1] = {
  {1, 8, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable opencensus__proto__trace__v1__RateLimitingSampler_msg_init = {
  NULL,
  &opencensus_proto_trace_v1_RateLimitingSampler__fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_psv8_1bt},
  })
};

static const upb_MiniTable *messages_layout[4] = {
  &opencensus__proto__trace__v1__TraceConfig_msg_init,
  &opencensus__proto__trace__v1__ProbabilitySampler_msg_init,
  &opencensus__proto__trace__v1__ConstantSampler_msg_init,
  &opencensus__proto__trace__v1__RateLimitingSampler_msg_init,
};

const upb_MiniTableFile opencensus_proto_trace_v1_trace_config_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  4,
  0,
  0,
};

#include "upb/port/undef.inc"

