/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/bootstrap/v3/bootstrap.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_CONFIG_BOOTSTRAP_V3_BOOTSTRAP_PROTO_UPB_MINITABLE_H_
#define ENVOY_CONFIG_BOOTSTRAP_V3_BOOTSTRAP_PROTO_UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__config__bootstrap__v3__Bootstrap_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Bootstrap__StaticResources_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Bootstrap__DynamicResources_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Bootstrap__ApplicationLogConfig_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Bootstrap__ApplicationLogConfig__LogFormat_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Bootstrap__DeferredStatOptions_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Bootstrap__GrpcAsyncClientManagerConfig_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Bootstrap__CertificateProviderInstancesEntry_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Admin_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__ClusterManager_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__ClusterManager__OutlierDetection_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Watchdogs_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Watchdog_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Watchdog__WatchdogAction_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__FatalAction_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__Runtime_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__RuntimeLayer_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__RuntimeLayer__DiskLayer_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__RuntimeLayer__AdminLayer_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__RuntimeLayer__RtdsLayer_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__LayeredRuntime_msg_init;
extern const upb_MiniTable envoy__config__bootstrap__v3__CustomInlineHeader_msg_init;

extern const upb_MiniTableFile envoy_config_bootstrap_v3_bootstrap_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_CONFIG_BOOTSTRAP_V3_BOOTSTRAP_PROTO_UPB_MINITABLE_H_ */
