/* This file was generated by upb_generator from the input file:
 *
 *     envoy/type/v3/http.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "envoy/type/v3/http.upb_minitable.h"
#include "udpa/annotations/status.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

const upb_MiniTableFile envoy_type_v3_http_proto_upb_file_layout = {
  NULL,
  NULL,
  NULL,
  0,
  0,
  0,
};

#include "upb/port/undef.inc"

