/* This file was generated by upb_generator from the input file:
 *
 *     xds/type/matcher/v3/http_inputs.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "xds/type/matcher/v3/http_inputs.upb_minitable.h"
#include "xds/annotations/v3/status.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

const upb_MiniTable xds__type__matcher__v3__HttpAttributesCelMatchInput_msg_init = {
  NULL,
  NULL,
  8, 0, kUpb_ExtMode_NonExtendable, 0, UPB_FASTTABLE_MASK(255), 0,
};

static const upb_MiniTable *messages_layout[1] = {
  &xds__type__matcher__v3__HttpAttributesCelMatchInput_msg_init,
};

const upb_MiniTableFile xds_type_matcher_v3_http_inputs_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  1,
  0,
  0,
};

#include "upb/port/undef.inc"

