/* This file was generated by upb_generator from the input file:
 *
 *     envoy/type/v3/ratelimit_unit.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_TYPE_V3_RATELIMIT_UNIT_PROTO_UPB_H_
#define ENVOY_TYPE_V3_RATELIMIT_UNIT_PROTO_UPB_H_

#include "upb/generated_code_support.h"

#include "envoy/type/v3/ratelimit_unit.upb_minitable.h"

#include "udpa/annotations/status.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
  envoy_type_v3_UNKNOWN = 0,
  envoy_type_v3_SECOND = 1,
  envoy_type_v3_MINUTE = 2,
  envoy_type_v3_HOUR = 3,
  envoy_type_v3_DAY = 4,
  envoy_type_v3_MONTH = 5,
  envoy_type_v3_YEAR = 6
} envoy_type_v3_RateLimitUnit;



#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_TYPE_V3_RATELIMIT_UNIT_PROTO_UPB_H_ */
