/* This file was generated by upb_generator from the input file:
 *
 *     src/proto/grpc/lb/v1/load_balancer.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef SRC_PROTO_GRPC_LB_V1_LOAD_BALANCER_PROTO_UPB_MINITABLE_H_
#define SRC_PROTO_GRPC_LB_V1_LOAD_BALANCER_PROTO_UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable grpc__lb__v1__LoadBalanceRequest_msg_init;
extern const upb_MiniTable grpc__lb__v1__InitialLoadBalanceRequest_msg_init;
extern const upb_MiniTable grpc__lb__v1__ClientStatsPerToken_msg_init;
extern const upb_MiniTable grpc__lb__v1__ClientStats_msg_init;
extern const upb_MiniTable grpc__lb__v1__LoadBalanceResponse_msg_init;
extern const upb_MiniTable grpc__lb__v1__FallbackResponse_msg_init;
extern const upb_MiniTable grpc__lb__v1__InitialLoadBalanceResponse_msg_init;
extern const upb_MiniTable grpc__lb__v1__ServerList_msg_init;
extern const upb_MiniTable grpc__lb__v1__Server_msg_init;

extern const upb_MiniTableFile src_proto_grpc_lb_v1_load_balancer_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* SRC_PROTO_GRPC_LB_V1_LOAD_BALANCER_PROTO_UPB_MINITABLE_H_ */
