/* This file was generated by upb_generator from the input file:
 *
 *     opencensus/proto/trace/v1/trace_config.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef OPENCENSUS_PROTO_TRACE_V1_TRACE_CONFIG_PROTO_UPB_MINITABLE_H_
#define OPENCENSUS_PROTO_TRACE_V1_TRACE_CONFIG_PROTO_UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable opencensus__proto__trace__v1__TraceConfig_msg_init;
extern const upb_MiniTable opencensus__proto__trace__v1__ProbabilitySampler_msg_init;
extern const upb_MiniTable opencensus__proto__trace__v1__ConstantSampler_msg_init;
extern const upb_MiniTable opencensus__proto__trace__v1__RateLimitingSampler_msg_init;

extern const upb_MiniTableFile opencensus_proto_trace_v1_trace_config_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* OPENCENSUS_PROTO_TRACE_V1_TRACE_CONFIG_PROTO_UPB_MINITABLE_H_ */
