/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/overload/v3/overload.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_CONFIG_OVERLOAD_V3_OVERLOAD_PROTO_UPB_MINITABLE_H_
#define ENVOY_CONFIG_OVERLOAD_V3_OVERLOAD_PROTO_UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__config__overload__v3__ResourceMonitor_msg_init;
extern const upb_MiniTable envoy__config__overload__v3__ThresholdTrigger_msg_init;
extern const upb_MiniTable envoy__config__overload__v3__ScaledTrigger_msg_init;
extern const upb_MiniTable envoy__config__overload__v3__Trigger_msg_init;
extern const upb_MiniTable envoy__config__overload__v3__ScaleTimersOverloadActionConfig_msg_init;
extern const upb_MiniTable envoy__config__overload__v3__ScaleTimersOverloadActionConfig__ScaleTimer_msg_init;
extern const upb_MiniTable envoy__config__overload__v3__OverloadAction_msg_init;
extern const upb_MiniTable envoy__config__overload__v3__LoadShedPoint_msg_init;
extern const upb_MiniTable envoy__config__overload__v3__BufferFactoryConfig_msg_init;
extern const upb_MiniTable envoy__config__overload__v3__OverloadManager_msg_init;

extern const upb_MiniTableFile envoy_config_overload_v3_overload_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_CONFIG_OVERLOAD_V3_OVERLOAD_PROTO_UPB_MINITABLE_H_ */
