/* This file was generated by upb_generator from the input file:
 *
 *     envoy/admin/v3/config_dump_shared.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_ADMIN_V3_CONFIG_DUMP_SHARED_PROTO_UPB_MINITABLE_H_
#define ENVOY_ADMIN_V3_CONFIG_DUMP_SHARED_PROTO_UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__admin__v3__UpdateFailureState_msg_init;
extern const upb_MiniTable envoy__admin__v3__ListenersConfigDump_msg_init;
extern const upb_MiniTable envoy__admin__v3__ListenersConfigDump__StaticListener_msg_init;
extern const upb_MiniTable envoy__admin__v3__ListenersConfigDump__DynamicListenerState_msg_init;
extern const upb_MiniTable envoy__admin__v3__ListenersConfigDump__DynamicListener_msg_init;
extern const upb_MiniTable envoy__admin__v3__ClustersConfigDump_msg_init;
extern const upb_MiniTable envoy__admin__v3__ClustersConfigDump__StaticCluster_msg_init;
extern const upb_MiniTable envoy__admin__v3__ClustersConfigDump__DynamicCluster_msg_init;
extern const upb_MiniTable envoy__admin__v3__RoutesConfigDump_msg_init;
extern const upb_MiniTable envoy__admin__v3__RoutesConfigDump__StaticRouteConfig_msg_init;
extern const upb_MiniTable envoy__admin__v3__RoutesConfigDump__DynamicRouteConfig_msg_init;
extern const upb_MiniTable envoy__admin__v3__ScopedRoutesConfigDump_msg_init;
extern const upb_MiniTable envoy__admin__v3__ScopedRoutesConfigDump__InlineScopedRouteConfigs_msg_init;
extern const upb_MiniTable envoy__admin__v3__ScopedRoutesConfigDump__DynamicScopedRouteConfigs_msg_init;
extern const upb_MiniTable envoy__admin__v3__EndpointsConfigDump_msg_init;
extern const upb_MiniTable envoy__admin__v3__EndpointsConfigDump__StaticEndpointConfig_msg_init;
extern const upb_MiniTable envoy__admin__v3__EndpointsConfigDump__DynamicEndpointConfig_msg_init;
extern const upb_MiniTable envoy__admin__v3__EcdsConfigDump_msg_init;
extern const upb_MiniTable envoy__admin__v3__EcdsConfigDump__EcdsFilterConfig_msg_init;

extern const upb_MiniTableFile envoy_admin_v3_config_dump_shared_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_ADMIN_V3_CONFIG_DUMP_SHARED_PROTO_UPB_MINITABLE_H_ */
