/* This file was generated by upb_generator from the input file:
 *
 *     envoy/data/accesslog/v3/accesslog.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_DATA_ACCESSLOG_V3_ACCESSLOG_PROTO_UPB_MINITABLE_H_
#define ENVOY_DATA_ACCESSLOG_V3_ACCESSLOG_PROTO_UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__data__accesslog__v3__TCPAccessLogEntry_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__HTTPAccessLogEntry_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__ConnectionProperties_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__AccessLogCommon_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__AccessLogCommon__FilterStateObjectsEntry_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__AccessLogCommon__CustomTagsEntry_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__ResponseFlags_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__ResponseFlags__Unauthorized_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__TLSProperties_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__TLSProperties__CertificateProperties_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__TLSProperties__CertificateProperties__SubjectAltName_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__HTTPRequestProperties_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__HTTPRequestProperties__RequestHeadersEntry_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__HTTPResponseProperties_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__HTTPResponseProperties__ResponseHeadersEntry_msg_init;
extern const upb_MiniTable envoy__data__accesslog__v3__HTTPResponseProperties__ResponseTrailersEntry_msg_init;

extern const upb_MiniTableFile envoy_data_accesslog_v3_accesslog_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_DATA_ACCESSLOG_V3_ACCESSLOG_PROTO_UPB_MINITABLE_H_ */
