/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/common/matcher/v3/matcher.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_CONFIG_COMMON_MATCHER_V3_MATCHER_PROTO_UPB_MINITABLE_H_
#define ENVOY_CONFIG_COMMON_MATCHER_V3_MATCHER_PROTO_UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__config__common__matcher__v3__Matcher_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__Matcher__OnMatch_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__Matcher__MatcherList_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__Matcher__MatcherList__Predicate_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__Matcher__MatcherList__Predicate__SinglePredicate_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__Matcher__MatcherList__Predicate__PredicateList_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__Matcher__MatcherList__FieldMatcher_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__Matcher__MatcherTree_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__Matcher__MatcherTree__MatchMap_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__Matcher__MatcherTree__MatchMap__MapEntry_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__MatchPredicate_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__MatchPredicate__MatchSet_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__HttpHeadersMatch_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__HttpGenericBodyMatch_msg_init;
extern const upb_MiniTable envoy__config__common__matcher__v3__HttpGenericBodyMatch__GenericTextMatch_msg_init;

extern const upb_MiniTableFile envoy_config_common_matcher_v3_matcher_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_CONFIG_COMMON_MATCHER_V3_MATCHER_PROTO_UPB_MINITABLE_H_ */
